/*
 * Copyright (c) Meta Platforms, Inc. and affiliates.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#pragma once

#include "cachelib/common/Serialization.h"
#include "cachelib/navy/serialization/gen-cpp2/objects_types.h"
#include "thrift/lib/cpp2/protocol/Serializer.h"

namespace facebook {
namespace cachelib {
namespace navy {
using ProtoSerializer = apache::thrift::BinarySerializer;

// @param obj       Object to be serialized. Must be a thrift object
// @param writer    Serializer that implements RecordWriter interface
template <typename ThriftObject>
void serializeProto(const ThriftObject& obj, RecordWriter& writer) {
  facebook::cachelib::serializeProto<ThriftObject, ProtoSerializer>(obj,
                                                                    writer);
}

// @param reader    Deserializer that implements RecordReader interface
// @return  deserialized thrift object
template <typename ThriftObject>
ThriftObject deserializeProto(RecordReader& reader) {
  return facebook::cachelib::deserializeProto<ThriftObject, ProtoSerializer>(
      reader);
}

} // namespace navy
} // namespace cachelib
} // namespace facebook
